USE omop4;
GO

/* *** BLOCK 1: Extract relevant data element FROM source data and store in #tmp *** */
IF OBJECT_ID('tempdb..#tmp') IS NOT NULL
	DROP TABLE #tmp;
CREATE TABLE #tmp (
	person_id INT
	,eventDate DATE
	,eventConceptId VARCHAR(100)
	,eventType VARCHAR(100)
	,eventNumValue FLOAT NULL
	,eventStringValue VARCHAR(100) NULL
	);
INSERT INTO #tmp
SELECT DISTINCT *
FROM (
	SELECT P0.person_id
		,U.eventDate
		,U.eventConceptId
		,U.eventType
		,U.eventNumValue
		,U.eventStringValue
	FROM omop4.eMergeCohort P0
	LEFT JOIN (
		SELECT person_id AS person_id
		, CONVERT(varchar,year_of_birth)+'-'
						+ CONVERT(varchar, month_of_birth) +'-' 
						+CONVERT(varchar, day_of_birth)  AS eventDate
		,NULL AS eventConceptId
		,'Birthday' AS eventType
		,NULL AS eventNumValue
		,NULL AS eventStringValue
	    FROM omop4.dbo.person
    
	    UNION ALL
    	SELECT person_id AS person_id
			,condition_start_date AS eventDate
			,condition_concept_id AS eventConceptId
			,'DxDiverticulitis' AS eventType
			,NULL AS eventNumValue
		    ,NULL AS eventStringValue
		FROM omop4.dbo.condition_occurrence
		WHERE condition_concept_id IN ('21009772','77025','193239','35707700','4048064','35707701')
		UNION ALL
    	SELECT person_id AS person_id
			,condition_start_date AS eventDate
			,condition_concept_id AS eventConceptId
			,'DxGIbleedDiverticRelated' AS eventType
			,NULL AS eventNumValue
		    ,NULL AS eventStringValue
		FROM omop4.dbo.condition_occurrence
		WHERE condition_concept_id IN ('21009772','77025','81611','194395','4048064','35707695','35707701')
		
		UNION ALL
		SELECT person_id AS person_id
			,procedure_date AS eventDate
			,procedure_concept_id AS eventConceptId
			,'ProcDiverticulosisColonoscopy' AS eventType
			,NULL AS eventNumValue
		    ,NULL AS eventStringValue
		FROM omop4.dbo.procedure_occurrence
		WHERE procedure_concept_id in ('2109193','2109199','2109194','2109200','2109201','2109197','2109196','2109195','2109205','2109203','2109202','2109204','2002705','2002703','2006781','40756787')
		UNION ALL
		SELECT person_id AS person_id
			,procedure_date AS eventDate
			,procedure_concept_id AS eventConceptId
			,'ProcDiverticulosisCT' AS eventType
			,NULL AS eventNumValue
		    ,NULL AS eventStringValue
		FROM omop4.dbo.procedure_occurrence
		WHERE procedure_concept_id in ('2211510','2211511','40757081','40756931','40756913','40756956','40756910','40757125','2211512')
		UNION ALL
		SELECT person_id AS person_id
			,procedure_date AS eventDate
			,procedure_concept_id AS eventConceptId
			,'ProcColectomy' AS eventType
			,NULL AS eventNumValue
		    ,NULL AS eventStringValue
		FROM omop4.dbo.procedure_occurrence
		WHERE procedure_concept_id in ('2109055','2109072','2109067','2109069','2109071','2109068','2109066','2109073','2109065','2109054')
		
		UNION ALL
		SELECT person_id AS person_id
			,observation_date AS eventDate
			,observation_concept_id AS eventConceptId
			,'LabBmi' AS eventType
			,value_as_number AS eventNumValue
		    ,value_as_string AS eventStringValue
		FROM omop4.dbo.observation
		WHERE observation_concept_id IN ('3038553') AND value_as_number IS NOT NULL
			AND ISNUMERIC(value_as_number) =1
		
		/* NLP part */
	  	UNION ALL
		SELECT person_id AS person_id
			,noteDate AS eventDate
			,NULL AS eventConceptId
			,'PathDiverticulitis' AS eventType
			,NULL AS eventNumValue
		    ,NULL AS eventStringValue
		FROM omop4.dbo.nlpPath
		WHERE AlgVar = 'diverticulitis.colonoscopy.mention'
		UNION ALL
		SELECT person_id AS person_id
			,noteDate AS eventDate
			,NULL AS eventConceptId
			,'PathDiverticulosis' AS eventType
			,NULL AS eventNumValue
		    ,NULL AS eventStringValue
		FROM omop4.dbo.nlpPath
		WHERE AlgVar = 'diverticulosis.colonoscopy.mention'
		
		) U ON P0.person_id = U.person_id
	) Utotal;

/* *** BLOCK 2: Get algorithm related variables *** */
IF OBJECT_ID('tempdb..#AlgVar') IS NOT NULL
	DROP TABLE #AlgVar;

CREATE TABLE #AlgVar (
	person_id INT NOT NULL
	,diseaseName VARCHAR(50) NOT NULL
	,caseControl VARCHAR(50) NOT NULL
	,C00 INT NOT NULL
	,C01 INT NOT NULL
	,C02 INT NOT NULL
	,C03 INT NOT NULL
	);

INSERT INTO #AlgVar
SELECT DISTINCT J.person_id
	,'Diverticulosis' AS diseaseName
	,'CASE/CONTROL' AS caseControl
	,ISNULL(J0.procColonoscopyCTcnt,0) AS C00
	,ISNULL(J1.dxDiverticulosisCnt, 0) AS C01
	,ISNULL(J2.dxDiverticulitisCnt, 0) AS C02
	,ISNULL(J3.procColonoscopyCnt,0) AS C03
	
FROM omop4.eMergeCohort J

LEFT JOIN (
	SELECT person_id
		,count(distinct eventDate) AS procColonoscopyCTcnt
	FROM #tmp
	WHERE eventType IN ('ProcDiverticulosisCT', 'ProcDiverticulosisColonoscopy')
	GROUP BY person_id
	) J0 
ON J.person_id = J0.person_id

LEFT JOIN (
	SELECT person_id
		,count(distinct eventDate) AS dxDiverticulosisCnt
	FROM #tmp
	WHERE eventType IN ('DxDiverticulitis', 'PathDiverticulosis')
	GROUP BY person_id
	) J1 
ON J.person_id = J1.person_id
LEFT JOIN (
	SELECT person_id
		,count(distinct eventDate) AS dxDiverticulitisCnt
	FROM #tmp
	WHERE eventType IN ('DxDiverticulitis', 'PathDiverticulitis')
	GROUP BY person_id
	) J2 
ON J.person_id = J2.person_id

LEFT JOIN (
	SELECT person_id
		,count(distinct eventDate) AS procColonoscopyCnt
	FROM #tmp
	WHERE eventType IN ('ProcDiverticulosisCT', 'ProcDiverticulosisColonoscopy')
	GROUP BY person_id
	) J3
ON J.person_id = J3.person_id
;

/* *** BLOCK 3: Phenotyping **** */
IF OBJECT_ID('tempdb..#phenotype') IS NOT NULL
	DROP TABLE #phenotype;
CREATE TABLE #phenotype (
	person_id INT
	,pCase VARCHAR(50)
	,pCtrl VARCHAR(50)
	);
INSERT INTO #phenotype
SELECT DISTINCT CL.person_id /* CL has duplicate person_id */
	,ISNULL(U1.phenotype, 'NO') AS pCase
	,ISNULL(U2.phenotype, 'NO') AS pCtrl
FROM omop4.eMergeCohort CL
LEFT JOIN (
	SELECT DISTINCT person_id
		,'YES' AS phenotype
	FROM #AlgVar
	WHERE (
			C00 > 0
			AND C01 > 0
			)
	) U1 ON CL.person_id = U1.person_id
LEFT JOIN (
	SELECT DISTINCT person_id
		,'YES' AS phenotype
	FROM #AlgVar
	WHERE (
			C03 > 0
			AND C01 = 0
			AND C02 = 0
			)
	) U2 ON CL.person_id = U2.person_id;

/* *** BLOCK 4: Write each patient phenotyping result (PR) to a table *** */
IF COL_LENGTH('$tablePR', 'DiverticulosisCase') IS NULL
	ALTER TABLE $tablePR ADD DiverticulosisCase VARCHAR(50);
IF COL_LENGTH('$tablePR', 'DiverticulosisCtrl') IS NULL
	ALTER TABLE $tablePR ADD DiverticulosisCtrl VARCHAR(50);

UPDATE $tablePR
SET DiverticulosisCase = (
		SELECT pCase
		FROM #phenotype
		WHERE #phenotype.person_id = $tablePR.person_id
		)
	,DiverticulosisCtrl = (
		SELECT pCtrl
		FROM #phenotype
		WHERE #phenotype.person_id = $tablePR.person_id
		);

/* *** BLOCK 5: Get count for each phenotype *** */
SELECT U1.CaseCnt
	,U2.CtrlCnt
FROM (
	SELECT Count(DISTINCT person_id) AS CaseCnt
	FROM #phenotype
	WHERE pCase = 'YES'
	) U1
	,(
		SELECT Count(DISTINCT person_id) AS CtrlCnt
		FROM #phenotype
		WHERE pCtrl = 'YES'
		) U2;

